local Buggers = Buggers
local L = LibStub("AceLocale-3.0"):GetLocale("Buggers", false)

Buggers.Option = {	
	type="group",
	width = 'fill',
	handler = Buggers,
	args={
	},
}

function Buggers:UpdateOptions()
	local mounts = {}
	for k in pairs(self.db.char.buggers) do
		mounts[#mounts + 1] = k
	end
	table.sort(mounts)
	for i, mountname in ipairs(mounts) do
		self.Option.args[tostring(i)] = self:CreateRange(mountname, i)
	end
end

function Buggers:GetRange(info)
	return self.db.char.buggers[info.option.name]
end

function Buggers:SetRange(info, num)
	self.db.char.buggers[info.option.name] = num
end

function Buggers:CreateRange(mountname,ordernum)
	local Range = {
			type = 'range',
			name = mountname,
			order = ordernum,
			width = 'full',
			min = 0,
			max = 10,
			step = 1,
			get = 'GetRange',
			set = 'SetRange',
		}
	return Range
end

Buggers.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('Buggers', Buggers.Option)
Buggers.optframe.Buggers = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('Buggers', 'Buggers')
-- function addapted from the RegisterModuleOptions function in Mapster
function Buggers:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "Buggers"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "Buggers")
end

do -- combat macro ahoy
	local db = {
		["char"] = L['char'],
		["class"] = L['class'],
		["race"] = L['race'],
		['global'] = L['global'],
	}
	
	local MacroOptions = {
		type="group",
		width = 'fill',
		handler = Buggers,
		childGroups = 'tab',
		args={
			Enable={
				type = 'toggle',
				order = 1,
				name = L['Enable'],
				desc = L['Enabledesc'],
				get = 'OGet',
				set = 'OSet',
			},
			select={
				name=L['Select'],
				type="select",
				order = 10,
				values = function(info)
					local v = {}
					for k in pairs(Buggers.db.global.macros) do
						if k == 'Default' then
							v[k] = L[k]
						else
							v[k] = k
						end
					end
					v['new'] = L["New"]
					return v
				end,
				get = 'OGet',
				set = 'OSet',
			},
			name={
				name=L['Name'],
				type="input",
				order = 20,
				get = 'OGet',
				set = 'OSet',
				validate = function(info,text)
					if Buggers.db.global.macros[text] then
						Buggers:Print(string.format(L["NameError"], text))
						return string.format(L["NameError"], text)
					else
						return true
					end
				end,
				disabled = 'ODisabled',
			},
			macro={
				name=ENTER_MACRO_LABEL,
				desc=L["FAQdesc"],
				type="input",
				multiline  = true,
				width = 'full',
				order = 30,
				get = 'OGet',
				set = 'OSet',
				validate = 'Ovalidate',
				disabled = 'ODisabled',
			},
			preview={
				type = 'toggle',
				order = 40,
				name = L['Preview'],
				desc = L['Toggle Preview'],
				get = 'OGet',
				set = 'OSet',
			},
			linebreak1 = {
				name = '',
				order = 45,
				type="description",
			},
			set = {
				name=L['Set Macro'],
				desc=L['Will set the macro to the selected one.\nWill also save the macro.'],
				type="execute",
				order = 50,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			reset = {
				name=L['Reset Macro'],
				desc=L['Will discard all changes and reset the macro to the previous one.'],
				type="execute",
				order = 60,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			delete = {
				name=L['Delete Macro'],
				desc=L['Will delete the selected macro and revert to the default macro.'],
				type="execute",
				order = 70,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			linebreak2 = {
				name = '',
				order = 80,
				type="description",
			},
			db={
				name=L['db'],
				type="select",
				order = 90,
				values = db,
				get = 'OGet',
				set = 'OSet',
			},
		},
	}
	
	function Buggers:OGet(info)
		if info[#info] == 'select' or info[#info] == 'name' then
			if not self.name then
				self.name = self.db[self.db.char.cmloc]
			end
			return self.name
		elseif info[#info] == 'db' then
			return self.db.char.cmloc
		elseif info[#info] == 'macro' then
			if not self.macro then
				self.macro = self.db.global.macros[self.db[self.db.char.cmloc]]
			end
			if self.preview then
				return self:ParseMacro(self.macro, true)
			end
			return self.macro
		elseif info[#info] == 'preview' then
			return self.preview
		elseif info[#info] == 'Enable' then
			return self.db.char.cmenabled
		end
	end

	function Buggers:OSet(info, key)
		if info[#info] == 'select' then
			if key == 'new' then
				self.name = ''
				self.macro = ''
				self.preview = nil
			else
				self.name = key
				self.macro = self.db.global.macros[key]
			end
		elseif info[#info] == 'name' then
			self.name = key
		elseif info[#info] == 'db' then
			if not self.db[key] then
				self.db[key] = self.db[self.db.charcmloc]
			end
			self.db.char.cmloc = key
		elseif info[#info] == 'macro' then
			if not self.locked[self.name] and not self.preview then
				self.macro = key
			end
		elseif info[#info] == 'preview' then
			self.preview = key
		elseif info[#info] == 'Enable' then
			self.db.char.cmenabled = key
		end
	end

	function Buggers:Ofunc(info)
		if info[#info] == 'set' then
			self.db.global.macros[self.name] = self.macro
			self.db[self.db.char[info[#info-1]..'loc']] = self.name
			self:SelectMacro(true)
		elseif info[#info] == 'reset' then
			self.name = nil
			self.macro = nil
		elseif info[#info] == 'delete' then
			if self.db[self.db.char[info[#info-1]..'loc']] == self.name then
				self.db[self.db.char[info[#info-1]..'loc']] = 'Default'
			end
			self.db.global.macros[self.name] = nil
			self.name = nil
			self.macro = nil
		end
	end

	function Buggers:Ovalidate(info, macro)
		for i=1,10 do
			if string.len(self:ParseMacro(macro, true)) > 255 then
				self:Print(L["Macro length error"])
				return L["Macro length error"]
			end
		end
		return true
	end

	function Buggers:ODisabled(info)
		if info[#info] == 'name' or info[#info] == 'delete' then
			if self.locked[self.name] then
				return true
			end
		elseif info[#info] == 'macro' then
			if self.locked[self.name] or self.preview then
				return true
			end
		elseif info[#info] == 'set' then
			if self.name == '' or self.macro == '' or self.db[self.db.char[info[#info-1]..'loc']] == self.name then
				return true
			end
		elseif info[#info] == 'reset' then
			if self.db[self.db.char[info[#info-1]..'loc']] == self.name then
				return true
			end
		end
		return nil
	end

	Buggers:RegisterModuleOptions(L['CombatMacro'], MacroOptions, L['CombatMacro'])
end


